@@ -2,7 +2,7 @@ use warnings;
 use strict;
 
 package Template::Declare::Buffer;
-use base 'Class::Accessor';
+use base 'Class::Accessor::Fast';
 
 __PACKAGE__->mk_accessors('data');
 
@@ -20,3 +20,17 @@ sub clear {
 }
 
 1;
+__END__
+
+=head1 NAME
+
+Template::Declare::Buffer
+
+=head1 DESCRIPTION
+
+We use this class to manage the output buffer used by L<Template::Declare>.
+
+=head1 SEE ALSO
+
+L<Template::Declare>.
+
